An instruction set is actually a whole set of configurations, usually contained in their own file and included in the avida.cfg file with <code>#include INST_SET=instset.cfg</code>.  

<h2>Specifiying Instruction Sets in avida.cfg</h2>

<p>Instruction sets are actually part of the avida.cfg file.  However, instead of writing the instruction set out in avida.cfg itself, which would be rather long and messy, each instruction set is placed in a separate file.  A preprocessor include (of the form <code>#include $filename</code>) is then used to include each instruction set in avida.cfg.  Multiple instruction sets may be included.  
</p>
<p>For example, to include the two instruction sets contained in the files instset-heads.cfg and instset-heads-sex.cfg, these lines are placed in avida.cfg:
</p>
<p>
<code>#include instset-heads.cfg</code><br />
<code>#include instset-heads-sex.cfg</code>
</p>
<p>You may optionally name each instruction set as you include it.  This allows you to change which file the name refers to when running avida from the command line, using the -def option.  The default avida.cfg contains this line:<br>
<code>#include INST_SET=instset-heads.cfg</code>
</p>
<p>This means you may change which file INST_SET refers (for example, change it to instset-heads-sex.cfg) to on the command line like this:<br>
<code>./avida -def INST_SET instset-heads-sex.cfg</code><br>
</p>
<p>In this case, INST_SET is simply a variable name with no special meaning.  The line might just as easily been <code>#include BOBS_YOUR_UNCLE=instset-heads.cfg</code></p>



<p>&nbsp;</p>
<h2>Specifiying the Instruction Set for an Organism</h2>

<p>Organisms in an avida run may have different instruction sets.  Therefore, it is necessary to specify the instruction used by each .org file.  At the top of the .org file you must specify the instruction set with the <code>#inst_set</code> keyword, and the hardware type with the <code>hw_type</code> keyword.  For example, default-heads.org starts with these two lines:
</p>
<p>
<code>#inst_set heads_default</code><br>
<code>#hw_type 0</code>
</p>
<p><b>Note:</b> the name of the instruction set may not be the same as the name of the file containing the instruction set.  (Indeed, one file may contain multiple instruction sets.)  The name of the instruction set is defined inside its .cfg file by the INSTSET keyword, which also defines the hardware type.



<p>&nbsp;</p>
<h2>The Structure of an Instruction Set File</h2>

<p>An instruction set consists of a name and hardware type, followed by a list of instructions that belong to
that instruction set, each of which may be followed by a series of options that
define how that instruction should be used.  The exact format is as follows:
</p>
<p>
<code>INSTSET instset-name:hw_type=$hardware_type</code><br />
<code>INST inst-name[:options]</code><br />
...
</p>

<p>Multiple instruction sets with unique names may be defined in the same file.  All <code>INST</code> statements must follow the
<code>INSTSET</code> statement defining the instruction set.</p>


<p>The following options may be specified (as <code>name=value</code>, separated by colons):
<dl>
<dt><strong>redundancy</strong> (default = 1)</dt>
<dd>
  The frequency of the instruction in the set.  One instruction with
  twice the redundancy of another with also have twice the probability
  of being mutated to.  A redundancy of zero is allowed, and indicates
  that injected organisms are allowed to have this instruction, but
  it can never be mutated to.
</dd>
<dt><strong>cost</strong></dt>
<dd>
  The number of CPU cycles required to execute this instruction.  One
  is the default if this value is not specified. CPU costs will pause the 
  thread that executed the costly instruction, not the whole organism and 
  only decrement the pause counter each time the paused thread attempts 
  to run. If multiple threads execute costly instructions, the costs and 
  pause counts are handled independently (i.e. unique pause count for each thread). 
</dd>
<dt><strong>initial_cost</strong></dt>
<dd>
  The additional cost to be paid the first time this instruction is
  executed.  This is used to lower the diversity of instructions 
  inside an organism.  The default value here is 0.
</dd>
<dt><strong>energy_cost</strong></dt>
<dd>
  The number of Energy units required to execute this instruction. Zero is the default if this value is not specified.
</dd>
<dt><strong>addl_time_cost</strong></dt>
<dd>
  The additional cost in terms of phenotype 'age', but not actual instruction cycles.  The default value here is 0.
</dd>
<dt><strong>prob_fail</strong>	
<dd>
  The probability of this instruction not working properly.  If an
  instruction fails it will simply do nothing, but still cost the
  CPU cycles to execute.  The defailt probability of failure is zero.
</dd>
<dt><strong>inst_code</strong>	
<dd>
  String of 1's and 0's defining the numeric value of a instruction as used by certain operations that 'numberate'
  instruction sequences.
</dd>
<dt><strong>res_cost</strong>	
<dd>
  The cost in units (double) of COLLECT_SPECIFIC_RESOURCE stored in internal bins to execute the instruction. Organisms 
  that execute an instruction with a res_cost higher than what is internally available will die. Res_cost is not 
  applied if an instruction fails. In a multi-resource environment, any and all resources can be converted internally 
  via reactions to a common COLLECT_SPECIFIC_RESOURCE for use with res_cost (COLLECT_SPECIFIC_RESOURCE must still be 
  declared in environment file, likely with initial=0). Note that the use of res_cost typically needs to be coupled with 
  RESOURCE_GIVEN_ON_INJECT and RESOURCE_GIVEN_AT_BIRTH in avida.cfg. The default res_cost is 0.
</dd>
<dt><strong>post_cost</strong>	
  <dd>
    CPU cost to be paid AFTER the instruction is executed the first time (e.g. pseudo post-kill handling time in 
    predators). Post costs are only applied if the instruction executes successfully (differs from regular costs).
  </dd>
<dt><strong>bonus_cost</strong>
<dd>
  The cost in units (double) of current bonus to successfully execute the instruction. bonus_cost is not
  applied if an instruction fails for some other reason. Ultimately this is similar to res_costs when converting a common,
  specific resource, but much simpiler.
</dd>
</dl>

<p>&nbsp;</p>
<p>For example, to specify that the nand instruction is both twice as common and twice as expensive as other instructions, you would use this line:</p>
<p><code>INST nand:redundancy=2:cost=2</code>


<p>&nbsp;</p>
<h2>Description of Default Instruction Set</h2>

<p>
Below are the descriptions of the instructions turned on in the file
<kbd>instset-classic.cfg</kbd>.  The one-letter codes are assigned
automatically to each instruction in the set, so if additional instructions
are turned on, the letters given below may no longer correspond to the
instructions they are presented with.  If more than 26 instructions are in
a set, both lowercase and capital letters will be used, and then numbers.
Currently, no more than 62 distinct instructions will be represented by
unique symbols.
</p>
<p>
Most terminology below that may not be familiar to you has been given a
link to a file containing its definition.
</p>

<h3>(a - c) Nop Instructions</h3>

The instructions <code>nop-A</code> (<b>a</b>), <code>nop-B</code>
(<b>b</b>), and <code>nop-C</code> (<b>c</b>) are no-operation
instructions, and will not do
anything when executed. They will, however, modifiy the behavior of the
instruction preceeding it (by changing the [CPU](Glossary-- CPU)
component that it affects; see also
<a href="Glossary--Nop-Register-Notation">nop-register</a>  notation and
<a href="Glossary--Nop-Head-Notation">nop-head</a>  notation) or act as part
of a <a href="Glossary--Label">label</a>  to denote positions in the
<a href="Glossary--Genome">genome</a>.


<h3>(d) <code>if-n-equ</code></h3>

This instruction compares the
<a href="Glossary-- Nop-Register-Notation">?BX?</a>  register to its
<a href="Glossary--Complement-Label">complement</a>. If they are not
equal, the next instruction (after a modifying
<a href="Glossary--nop-instructions">no-operation</a>  instruction, if one is
present) is executed. If they are equal, that next instruction is skipped.


<h3>(e) <code>if-less</code></h3>

This instruction compares the <a href="Glossary--Nop-Register-Notation">?BX?</a>
register to its <a href="Glossary--Complement-Label">complement</a>. If ?BX?
is the lesser of the pair, the next instruction (after a modifying
<a href="Glossary--nop-instructions">no-operation</a>  instruction, if one is
present) is executed. If it is greater or equal, then that next instruction is skipped.


<h3>(f) <code>pop</code></h3>

This instruction removes the top element from the active
<a href="Glossary--Stack">stack</a>, and places it into the
<a href="Glossary--Nop-Register-Notation">?BX?</a>  register.


<h3>(g) <code>push</code></h3>

This instruction reads in the contents of the
<a href="Glossary--Nop-Register-Notation">?BX?</a>  register, and places it
as a new entry at the top of the active <a href="Glossary--Stack">stack</a>.
The ?BX? register itself remains unchanged.


<h3>(h) <code>swap-stk</code></h3>

This instruction toggles the active <a href="Glossary--Stack">stack</a>  in
the <a href="Glossary--CPU">CPU</a>. All other instructions that use a stack
will always use the active one.


<h3>(i) <code>swap</code></h3>

This instruction swaps the contents of the
<a href="Glossary--Nop-Register-Notation">?BX?</a>  register with its
<a href="Glossary--Complement-Label">complement</a>.


<h3>(j) <code>shift-r</code></h3>

This instruction reads in the contents of the
<a href="Glossary--Nop-Register-Notation">?BX?</a>  register, and shifts all
of the bits in that register to the right by one. In effect, it divides the value
stored in the register by two, rounding down.


<h3>(k) <code>shift-l</code></h3>

This instruction reads in the contents of the
<a href="Glossary--Nop-Register-Notation">?BX?</a>  register, and shifts all
of the bits in that register to the left by one, placing a zero as the new rightmost
bit, and trunkating any bits beyond the 32 maximum. For values that require fewer
than 32 bits, it effectively multiplies that value by two.


<h3>(l) <code>inc</code> and (m) <code>dec</code></h3>

These instructions read in the contents of the
<a href="Glossary--Nop-Register-Notation">?BX?</a> 
<a href="Glossary--Registers">register</a> and increment or decrement it by
one.

<h3>(n) <code>add</code> and (o) <code>sub</code></h3>

These instructions read in the contents of the BX and CX
<a href="Glossary--Registers">registers</a> and either sums them together or
subtracts CX from BX (respectively). The result of this operation is then placed in
the <a href="Glossary--Nop-Register-Notation">?BX?</a>  register.

<h3>(p) <code>nand</code></h3>

This instruction reads in the contents of the BX and CX
<a href="Glossary--Registers">registers</a>  (each of which are 32-bit
numbers) and performs a <a href="Glossary--Bitwise">bitwise</a>  nand
operation on them. The result of this operation is placed in the
<a href="Glossary--Nop-Register-Notation">?BX?</a>  register. Note that
this is the only <a href="Glossary--Logic">logic</a>  operation provided in
the basic Avida instruction set.


<h3>(q) <code>IO</code></h3>

This is the input/output instruction. It takes the contents of the
<a href="Glossary--Nop-Register-Notation">?BX?</a>
<a href="Glossary--Registers">register</a>  and outputs it, checking it
for any <a href="Glossary--Tasks">tasks</a>  that may have been performed.
It will then place a new <a href="Glossary--Input-Output">input</a> into ?BX?.


<h3>(r) <code>h-alloc</code></h3>

This instruction allocates additional <a href="Glossary--Memory">memory</a>
for the organism up to the maximum it is allowed to use for its offspring. 
If allocation is successful, the original memory size is loaded into the AX register.


<h3>(s) <code>h-divide</code></h3>

This instruction is used for an organism to divide off an finnished offspring. The
original organism keeps the state of its <a href="Glossary--Memory">memory</a>
up until the <a href="Glossary--Heads">read-head</a>. The offspring's memory is
initialized to everything between the read-head and the
<a href="Glossary--Heads">write-head</a>. All memory past the write-head is
removed entirely.


<h3>(t) <code>h-copy</code></h3>

This instruction reads the contents of the organism's
<a href="Glossary--Memory">memory</a>  at the position of the
<a href="Glossary--Heads">read-head</a>, and copy that to the position of the
<a href="Glossary--Heads">write-head</a>. If a non-zero copy mutation rate is
set, a test will be made based on this probability to determine if a
<a href="Glossary--Mutation">mutation</a>  occurs. If so, a random instruction
(chosen from the full set with equal probability) will be placed at the write-head
instead.


<h3>(u) <code>h-search</code></h3>

This instruction will read in the <a href="Glossary--Label">label</a>
the follows it, and find the location of a
<a href="Glossary--Complement-Label">complement</a> label in the code.
The BX <a href="Glossary--Registers">register</a> will be set to the distance
to the complement from the current position of the
<a href="Glossary--Heads">instruction-pointer</a>, and the CX register will
be set to the size of the label. The <a href="Glossary--Heads">flow-head</a>
will also be placed at the beginning of the complement label. If no label follows,
both BX and CX will be set to zero, and the flow-head will be placed on the instruction
immediatly following the h-search.


<h3>(v) <code>mov-head</code></h3>

This instruction will cause the <a href="Glossary--Nop-Head-Notation">?IP?</a>
to jump to the position in <a href="Glossary--Memory">memory</a>  of the
<a href="Glossary--Heads">flow-head</a>.


<h3>(w) <code>jmp-head</code></h3>

This instruction will read in the value of the CX
<a href="Glossary--Registers">register</a>, and the move the
<a href="Glossary--Nop-Head-Notation">?IP?</a> by that fixed amount through
the organism's <a href="Glossary--Memory">memory</a>.


<h3>(x) <code>get-head</code></h3>

This instruction will copy the position of the
<a href="Glossary--Nop-Head-Notation">?IP?</a> into the CX
<a href="Glossary--Registers">register</a>.


<h3>(y) <code>if-label</code></h3>

This instruction reads in the <a href="Glossary--Label">label</a> that
follows it, and tests if its <a href="Glossary--Complement-Label">complement</a>
label was the most recent series of instructions copied. If so, it executed the next
instruction, otherwise it skips it. This instruction is commonly used for an organism to
determine when it has finished producing its offspring.


<h3>(z) <code>set-flow</code></h3>

This instruction moves the <a href="Glossary--Heads">flow-head</a> to the
<a href="Glossary--Memory">memory</a>  position denoted in the
<a href="Glossary--Nop-Register-Notation">?CX?</a>  register.
